package GT::Maildir::Folder;

use strict;
use warnings;
use base 'GT::Base';

use GT::Maildir qw/:stat/;
use GT::Dumper;

use Carp qw/croak/;


sub init {
    my $self = shift;

    my $path = shift;
    croak "No maildir path specified to $self->init" unless defined $path and length $path;

    my $folder = shift;
    croak "No folder specified to $self->init" unless defined $folder and length $folder;
    croak "Unknown arguments to $self->init: ", join(", ", @_) if @_;

    $self->{name} = $folder;
    $self->{path} = $path;
    $self->{data} = {
        name => $folder
    };
}

sub get_name {
    my $self = shift;
    croak "Unknown arguments to $self->get_name: ", join(", ", @_) if @_;
    return $self->{name};
}

sub get_path {
    my $self = shift;
    croak "Unknown arguments to $self->get_new_path: ", join(", ", @_) if @_;
    return "$self->{path}/$self->{name}";
}

sub get_new_path {
    my $self = shift;
    croak "Unknown arguments to $self->get_new_path: ", join(", ", @_) if @_;
    return "$self->{path}/$self->{name}/new";
}

sub get_cur_path {
    my $self = shift;
    croak "Unknown arguments to $self->get_cur_path: ", join(", ", @_) if @_;
    return "$self->{path}/$self->{name}/cur";
}

sub needs_update {
    my $self = shift;
    croak "Unknown arguments to $self->needs_update: ", join(", ", @_) if @_;
    my @st;
    @st = stat($self->get_new_path);
    if (!@st) {
        return 2;
    }
    if ($st[ST_MTIME] != $self->get('new_mtime')) {
        return 1;
    }
    @st = stat($self->get_cur_path);
    if (!@st) {
        return 2;
    }
    if ($st[ST_MTIME] != $self->get('cur_mtime')) {
        return 1;
    }
    return 0;
}

sub is_deleted {
    my $self = shift;
    croak "Unknown arguments to $self->is_deleted: ", join(", ", @_) if @_;
    return $self->get("deleted");
}

sub get {
    my $self = shift;
    my $key = shift;
    croak "No key specified to $self->get" unless defined $key and length $key;
    croak "Unknown arguments to $self->get: ", join(", ", @_) if @_;
    return $self->{data}{$key};
}

sub set {
    my $self = shift;
    my $key = shift;
    croak "No key specified to $self->set" unless defined $key and length $key;
    croak "No value specified to $self->set" unless @_;
    my $value = shift;
    croak "Unknown arguments to $self->set: ", join(", ", @_) if @_;
    $self->{data}{$key} = $value;
}

sub get_new_mtime {
    my $self = shift;
    croak "Unknown arguments to $self->get_new_mtime: ", join(", ", @_) if @_;
    return $self->get('new_mtime');
}

sub get_new_count {
    my $self = shift;
    croak "Unknown arguments to $self->get_new_count: ", join(", ", @_) if @_;
    return $self->get('new_count');
}

sub get_cur_mtime {
    my $self = shift;
    croak "Unknown arguments to $self->get_cur_mtime: ", join(", ", @_) if @_;
    return $self->get('cur_mtime');
}

sub get_cur_count {
    my $self = shift;
    croak "Unknown arguments to $self->get_cur_count: ", join(", ", @_) if @_;
    return $self->get('cur_count');
}

sub get_count {
    my $self = shift;
    croak "Unknown arguments to $self->get_count: ", join(", ", @_) if @_;
    return $self->get('cur_count') + $self->get('new_count');
}

sub data {
    my $self = shift;
    croak "Unknown arguments to $self->data: ", join(", ", @_) if @_;
    return {%{$self->{data}}};
}

sub update {
    my $self = shift;
    croak "Unknown arguments to $self->update: ", join(", ", @_) if @_;

    my $cur = $self->get_cur_path;
    my $new = $self->get_new_path;
    my $cur_mtime = (stat($cur))[ST_MTIME];
    my $new_mtime = (stat($new))[ST_MTIME];
    my ($new_count, $cur_count) = (0, 0);
    my $dir = do { local *FH; \*FH };
    if (opendir $dir, $cur) {
        # This could be a tight loop on a large folder..
        # Don't do any extra work in here unless it's really needed!
        while (my $f = readdir $dir) {
            next if $f eq '.' or $f eq '..';
            $cur_count++;
        }
        closedir $dir;
    }
    else {
        $self->error("OPENDIR", "WARN", $cur, "$!");
    }
    if (opendir $dir, $new) {
        # This could be a tight loop on a large folder..
        # Don't do any extra work in here unless it's really needed!
        while (my $f = readdir $dir) {
            next if $f eq '.' or $f eq '..';
            $new_count++;
        }
        closedir $dir;
    }
    else {
        $self->error("OPENDIR", "WARN", $new, "$!");
    }
    $self->set(cur_mtime => $cur_mtime);
    $self->set(cur_count => $cur_count);
    $self->set(new_mtime => $new_mtime);
    $self->set(new_count => $new_count);
    return 1;
}

1;
