package GT::Maildir::Lock::NFSLock;

use strict;
use warnings;
use base 'GT::Maildir::Lock';

use File::NFSLock;
use Carp qw/croak/;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_);
    $self->{locks} = {};
    return $self;
}

sub do_unlock {
    my ($self, $file) = @_;
    my $lock = delete $self->{locks}{$file};
    return undef unless $lock;
    $lock->unlock;
    return 1;
}

sub do_lock {
    my ($self, $file, $type, $blocking_timeout, $stale_lock_timeout) = @_;
    $self->{locks}{$file} = new File::NFSLock(
        $file, $type,
        $blocking_timeout, $stale_lock_timeout
    );
    if (!$self->{locks}{$file}) {
        $self->set_errorstr($File::NFSLock::errstr);
        return undef;
    }
    return 1;
}

sub DESTROY {
    my $self = shift;
    $_->unlock for values %{$self->{locks}};
    undef;
}

1;

