package GT::Maildir::Lucene::Indexer::Analyzer;

use vars qw/@STOPWORDS/;
use strict;

use GT::Maildir::Lucene qw/%Lucene_Table_Defs/;
use GT::Maildir::Lucene::Indexer::Tokenizer;

use Carp qw/confess/;
use Lucene;
use base 'Lucene::Analysis::Analyzer';

@STOPWORDS = qw/
    of about or all several also she among since an some and such are than
    as that at the be them because there been these between they both this
    but those by to do toward during towards each upon either for from was
    had were has what have when he where her which his while however with if
    within in would into you your is it its many more most must on re it
    test not above add am pm jan january feb february mar march apr april
    may jun june jul july aug august sep sept september oct october nov
    november dec december find &amp &gt &lt we http com www inc other
    including
/;


sub tokenStream {
    my ($self, $field, $reader) = @_;
    my $field_data = $Lucene_Table_Defs{$field};
    confess "No field data for $field" unless $field_data;
    # don't modify Keyword fields
    if ($field_data->{storage} eq 'Keyword') {
        return new GT::Maildir::Lucene::Indexer::Tokenizer($reader);
    }
    # This is just what Lucene::Analysis::StandardAnalyzer does
    my $ret = new Lucene::Analysis::StandardTokenizer($reader);
    $ret = new Lucene::Analysis::StandardFilter($ret);
    $ret = new Lucene::Analysis::LowerCaseFilter($ret);
    $ret = new Lucene::Analysis::StopFilter($ret, [@STOPWORDS]);
    return $ret;
}

1;
