package GT::Maildir::Search::SortField;

use strict;

use base 'GT::Maildir::Search::Sort';
use Carp qw/croak/;

use GT::Maildir::Lucene;

sub new {
	my $class = shift;

	croak "Invalid arguments to $class->new. Arguments must be key/value pairs" if @_ & 1;

	my %opts = @_;
	$opts{ lc $_ } = delete $opts{$_} for keys %opts;

	my $field = delete $opts{field};
	croak "No field specified to $class->new" unless $field;
	$field = lc $field;
	my $table_defs = GT::Maildir::Lucene->get_table_defs;
	if (exists $table_defs->{$field} and exists $table_defs->{$field}{sort_field}) {
		$field = $table_defs->{$field}{sort_field};
	}

	my $desc = delete $opts{descending};

	return bless {
		field => $field,
		desc => $desc,
        type => GT::Maildir::Search::Sort::ST_FIELD()
	};
}

sub get_field {
	my $self = shift;
	croak "Unknown arguments to $self->get_field" if @_;
	return $self->{field};
}

sub is_descending {
	my $self = shift;
	croak "Unknown arguments to $self->get_descending" if @_;
	return $self->{desc};
}

sub is_ascending {
	my $self = shift;
	croak "Unknown arguments to $self->get_descending" if @_;
	return !$self->{desc};
}

1;

